using CanonSharp.Combinator.Abstractions;

namespace CanonSharp.Combinator.Parsers.Modifiers;

/// <summary>
/// 对结果运行指定操作，但是不做修改操作的解析器
/// </summary>
/// <param name="parser">上游解析器</param>
/// <param name="succeed">对成功结果的操作</param>
/// <param name="fail">对失败结果的操作</param>
/// <typeparam name="TToken">输入流类型</typeparam>
/// <typeparam name="T">解析结果类型</typeparam>
internal sealed class DoParser<TToken, T>(
    IParser<TToken, T> parser,
    Action<T> succeed,
    Action<IFailedResult<TToken, T>> fail) : ModifiedParser<TToken, T, T>(parser)
{
    protected override IParseResult<TToken, T> Succeed<TState>(TState state,
        ISuccessfulResult<TToken, T> successfulResult)
    {
        succeed(successfulResult.Value);
        return successfulResult;
    }

    protected override IParseResult<TToken, T> Fail<TState>(TState state, IFailedResult<TToken, T> failedResult)
    {
        fail(failedResult);
        return failedResult;
    }
}
