using CanonSharp.Combinator.Abstractions;

namespace CanonSharp.Combinator.Parsers.Modifiers;

/// <summary>
/// 向前看解析器
/// 使用传入的解析器向前解析
/// 但是返回的结果中输入流读取状态不前移
/// </summary>
/// <param name="parser">需要向前看的解析器</param>
/// <typeparam name="TToken">输入流令牌</typeparam>
/// <typeparam name="T">返回的解析结果类型</typeparam>
internal sealed class LookAheadParser<TToken, T>(IParser<TToken, T> parser) : ModifiedParser<TToken, T, T>(parser)
{
    protected override IParseResult<TToken, T> Succeed<TState>(TState state,
        ISuccessfulResult<TToken, T> successfulResult)
        => ParseResultBuilder.Succeed<TToken, TState, T>(successfulResult.Value, state);

    protected override IParseResult<TToken, T> Fail<TState>(TState state, IFailedResult<TToken, T> failedResult)
        => ParseResultBuilder.Fail<TToken, TState, T>($"Failed when looking ahead: {failedResult}", state);
}
